@@ -1,17 +1,16 @@
 use {
     super::*,
     crate::{
+        cli::Args,
         core::OpenCloset,
         error::SafeClosetError,
         timer::Timer,
     },
     crossterm::event::KeyModifiers,
     crossbeam::select,
-    std::time::Duration,
     termimad::{Area, Event, EventSource},
 };
 
-const MAX_INACTIVITY: Duration = Duration::from_secs(45);
 
 /// Run the Terminal User Interface until the user decides to quit.
 ///
@@ -19,9 +18,9 @@ const MAX_INACTIVITY: Duration = Duration::from_secs(45);
 pub(super) fn run(
     w: &mut W,
     open_closet: OpenCloset,
-    hide_values: bool,
+    args: &Args,
 ) -> Result<(), SafeClosetError> {
-    let mut state = AppState::new(open_closet, hide_values);
+    let mut state = AppState::new(open_closet, args);
     let mut view = GlobalView::default();
     view.set_area(Area::full_screen());
     view.draw(w, &mut state)?;
@@ -50,6 +49,9 @@ pub(super) fn run(
                         state.on_click(x, y)?;
                         timer.reset();
                     }
+                    Event::Wheel(amount) => {
+                        state.on_mouse_wheel(amount);
+                    }
                     _ => {}
                 }
                 event_source.unblock(quit);
