@@ -1,6 +1,10 @@
 use {
     super::*,
-    crate::{core::*, error::SafeClosetError},
+    crate::{
+        cli::Args,
+        core::*,
+        error::SafeClosetError,
+    },
     crossterm::{self, event::KeyEvent},
 };
 
@@ -10,7 +14,9 @@ use {
 pub struct AppState {
     pub open_closet: OpenCloset,
     pub drawer_state: DrawerState,
-    pub error: Option<String>,
+    // the help state, if help is currently displayed
+    pub help: Option<HelpState>,
+    pub message: Option<Message>,
     pub hide_values: bool,
     // number of drawers created during this session
     pub created_drawers: usize,
@@ -18,16 +24,34 @@ pub struct AppState {
 
 impl AppState {
 
-    pub fn new(open_closet: OpenCloset, hide_values: bool) -> Self {
-        Self {
+    pub fn new(open_closet: OpenCloset, args: &Args) -> Self {
+        let mut state = Self {
             open_closet,
             drawer_state: DrawerState::NoneOpen,
-            error: None,
-            hide_values,
+            help: None,
+            message: None,
+            hide_values: args.hide,
             created_drawers: 0,
+        };
+        if args.open && !state.open_closet.just_created() {
+            state.drawer_state = DrawerState::DrawerOpening(PasswordInputState::new(true));
         }
+        state
     }
 
+    fn set_error<S: Into<String>>(&mut self, error: S) {
+        let text = error.into();
+        warn!("error: {:?}", &text);
+        self.message = Some(Message{ text, error: true });
+    }
+    #[allow(dead_code)]
+    fn set_info<S: Into<String>>(&mut self, info: S) {
+        let text = info.into();
+        debug!("info: {:?}", &text);
+        self.message = Some(Message{ text, error: false });
+    }
+
+
     /// If there's an open drawer input (entry name or value), close it, keeping
     /// the input content if required.
     ///
@@ -44,12 +68,13 @@ impl AppState {
     fn save(&mut self, reopen_if_open: bool) -> Result<(), SafeClosetError> {
         time!(self.close_drawer_input(false));
         let drawer_state = std::mem::take(&mut self.drawer_state);
-        if let DrawerState::DrawerEdit(des) = drawer_state {
+        if let DrawerState::DrawerEdit(mut des) = drawer_state {
             if reopen_if_open {
                 self.drawer_state = DrawerState::DrawerEdit(
                     time!(des.save_and_reopen(&mut self.open_closet)?)
                 );
             } else {
+                des.drawer.content.remove_empty_entries();
                 time!(self.open_closet.push_back(des.drawer)?);
                 time!(self.open_closet.close_and_save())?;
             }
@@ -57,6 +82,55 @@ impl AppState {
         Ok(())
     }
 
+    /// Handle an event asking for pasting into SafeCloset
+    pub fn paste(&mut self) {
+        #[cfg(not(feature = "clipboard"))]
+        {
+            self.set_error("Clipboard feature not enabled at compilation");
+        }
+        #[cfg(feature = "clipboard")]
+        {
+            use DrawerFocus::*;
+            match terminal_clipboard::get_string() {
+                Ok(mut pasted) if !pasted.is_empty() => {
+                    if !self.drawer_state.is_on_entry_value() {
+                        pasted.truncate(pasted.lines().next().unwrap().len());
+                    }
+                    if let Some(input) = self.drawer_state.input() {
+                        input.insert_str(pasted);
+                    } else if let DrawerState::DrawerEdit(des) = &mut self.drawer_state {
+                        if let NameSelected { line } = &mut des.focus {
+                            let line = *line;
+                            if des.edit_entry_name_by_line(line, EditionPos::Start) {
+                                if let Some(input) = self.drawer_state.input() {
+                                    input.set_str(pasted);
+                                    input.move_to_end();
+                                    self.set_info("Hit *esc* to cancel pasting");
+                                } else {
+                                    warn!("unexpected lack of input");
+                                }
+                            }
+                        } else if let ValueSelected { line } = &mut des.focus {
+                            let line = *line;
+                            if des.edit_entry_value_by_line(line, EditionPos::Start) {
+                                if let Some(input) = self.drawer_state.input() {
+                                    input.set_str(pasted);
+                                    input.move_to_end();
+                                    self.set_info("Hit *esc* to cancel pasting");
+                                } else {
+                                    warn!("unexpected lack of input");
+                                }
+                            }
+                        }
+                    }
+                }
+                _ => {
+                    self.set_error("nothing to paste");
+                }
+            }
+        }
+    }
+
     /// Handle a click event
     pub fn on_click(&mut self, x: u16, y: u16)-> Result<(), SafeClosetError> {
 
@@ -67,7 +141,6 @@ impl AppState {
                 return Ok(());
             } else if let DrawerState::DrawerEdit(des) = &mut self.drawer_state {
                 // unfocusing the input, validating it
-                debug!("unfocusing des input");
                 des.focus = DrawerFocus::NoneSelected;
             }
         }
@@ -87,6 +160,15 @@ impl AppState {
         Ok(())
     }
 
+    /// Handle a mouse wheel event
+    pub fn on_mouse_wheel(&mut self, amount: i32) {
+        if let DrawerState::DrawerEdit(des) = &mut self.drawer_state {
+            des.move_line(
+                if amount < 0 { Direction::Up } else { Direction::Down }
+            );
+        }
+    }
+
     /// push back the open drawer, if any, and set the drawer_state to NoneOpen
     fn push_back_drawer(&mut self) -> Result<(), SafeClosetError> {
         self.close_drawer_input(true);
@@ -98,20 +180,17 @@ impl AppState {
         Ok(())
     }
 
+
     /// Handle a key event
     pub fn on_key(&mut self, key: KeyEvent) -> Result<CmdResult, SafeClosetError> {
         use {
             DrawerFocus::*,
             DrawerState::*,
         };
-        self.error = None;
+        self.message = None;
 
-        if key == CONTROL_C { // close drawer (no save)
-            // we're not repushing the drawer, so we're effectively
-            // going up in the closet
-            self.drawer_state = DrawerState::NoneOpen;
-            return Ok(CmdResult::Stay);
-        }
+        // We start with the few actions that can be done the same with or
+        // without the help screen displayed
 
         if key == CONTROL_N { // new drawer
             self.push_back_drawer()?;
@@ -120,6 +199,7 @@ impl AppState {
         }
 
         if key == CONTROL_O { // open drawer
+            self.help = None;
             self.push_back_drawer()?;
             self.drawer_state = DrawerOpening(PasswordInputState::new(true));
             return Ok(CmdResult::Stay);
@@ -142,6 +222,48 @@ impl AppState {
             return Ok(CmdResult::Quit);
         }
 
+        if key == CONTROL_C { // close
+            if self.help.is_some() {
+                // close the help
+                self.help = None;
+            } else {
+                // close the drawer
+                self.save(true)?;
+                self.push_back_drawer()?;
+                let _ = self.open_closet.close_deepest_drawer();
+                self.drawer_state = match self.open_closet.take_deepest_open_drawer() {
+                    Some(open_drawer) => DrawerState::edit(open_drawer),
+                    None => DrawerState::NoneOpen,
+                };
+            }
+            return Ok(CmdResult::Stay);
+        }
+
+        if key == CONTROL_V {
+            self.paste();
+            return Ok(CmdResult::Stay);
+        }
+
+        if key == ESC {
+            if self.help.is_some() {
+                self.help = None;
+            } else if matches!(self.drawer_state, DrawerCreation(_) | DrawerOpening(_)) {
+                self.drawer_state = NoneOpen;
+            } else if let DrawerEdit(des) = &mut self.drawer_state {
+                if !des.close_input(true) {
+                    des.focus = NoneSelected;
+                }
+            }
+            return Ok(CmdResult::Stay);
+        }
+
+        // If the help is shown, it captures other events
+        if let Some(help_state) = &mut self.help {
+            help_state.apply_key_event(key);
+            return Ok(CmdResult::Stay);
+        }
+
+
         if key == CONTROL_UP { // moving the selected line up
             if let DrawerEdit(des) = &mut self.drawer_state {
                 des.close_input(false);
@@ -160,6 +282,7 @@ impl AppState {
                     }
                     _ => {}
                 }
+                des.update_search();
             }
             return Ok(CmdResult::Stay);
         }
@@ -181,6 +304,7 @@ impl AppState {
                     }
                     _ => {}
                 }
+                des.update_search();
             }
             return Ok(CmdResult::Stay);
         }
@@ -196,10 +320,11 @@ impl AppState {
                         info!("user requests entry removal");
                         des.drawer.content.entries.remove(idx);
                         des.focus = if line > 0 {
-                            NameSelected { line }
+                            NameSelected { line: line - 1 }
                         } else {
                             NoneSelected
                         };
+                        des.update_search();
                     } else {
                         info!("user cancels entry removal");
                         des.focus = NameSelected { line };
@@ -235,8 +360,7 @@ impl AppState {
                         self.created_drawers += 1;
                     }
                     Err(e) => {
-                        warn!("error in drawer creation: {}", e);
-                        self.error = Some(e.to_string());
+                        self.set_error(e.to_string());
                     }
                 }
             } else if let DrawerOpening(PasswordInputState { input }) = &mut self.drawer_state {
@@ -250,58 +374,54 @@ impl AppState {
                         self.drawer_state = DrawerState::edit(open_drawer);
                     }
                     None => {
-                        warn!("no drawer can be opened with this passphrase");
-                        self.error = Some("This passphrase opens no drawer".to_string());
+                        self.set_error("This passphrase opens no drawer");
                     }
                 }
             }
             return Ok(CmdResult::Stay);
         }
 
-        if key == ESC {
-            if matches!(self.drawer_state, DrawerCreation(_) | DrawerOpening(_)) {
-                self.drawer_state = NoneOpen;
-            } else if let DrawerEdit(des) = &mut self.drawer_state {
-                if !des.close_input(true) {
-                    des.focus = NoneSelected;
-                }
-            }
-            return Ok(CmdResult::Stay);
-        }
-
         if key == TAB {
             if let DrawerEdit(des) = &mut self.drawer_state {
                 if matches!(des.focus, NoneSelected) {
                     // we remove any search
                     des.search.clear();
                     let idx = des.drawer.content.empty_entry();
-                    des.edit_entry_name_by_line(idx); // as there's no filtering, idx==line
+                    des.edit_entry_name_by_line(idx, EditionPos::Start); // as there's no filtering, idx==line
                 } else if let NameSelected { line } = &des.focus {
                     let line = *line;
-                    des.edit_entry_value_by_line(line);
+                    des.edit_entry_value_by_line(line, EditionPos::Start);
                 } else if let NameEdit { line, .. } = &des.focus {
                     let line = *line;
                     des.close_input(false);
-                    des.edit_entry_value_by_line(line);
+                    des.edit_entry_value_by_line(line, EditionPos::Start);
                 } else if let ValueSelected { line } | ValueEdit { line, .. } = &des.focus {
                     let line = *line;
+                    des.close_input(false);
                     if des.listed_entries_count() == line + 1 {
                         // last listed entry
                         if des.drawer.content.entries[line].is_empty() {
                             // if the current entry is empty, we don't create a new one
                             // but go back to the current (empty) entry name
-                            des.edit_entry_name_by_line(line);
+                            des.edit_entry_name_by_line(line, EditionPos::Start);
                         } else {
                             // we create a new entry and start edit it
                             // but we must ensure there's no search which could filter it
                             des.search.clear();
                             des.drawer.content.entries.push(Entry::default());
-                            des.edit_entry_name_by_line(des.drawer.content.entries.len() - 1);
+                            des.edit_entry_name_by_line(
+                                des.drawer.content.entries.len() - 1,
+                                EditionPos::Start,
+                            );
                         }
                     } else {
-                        des.edit_entry_name_by_line(line + 1);
+                        des.edit_entry_name_by_line(
+                            line + 1,
+                            EditionPos::Start,
+                        );
                     }
                 }
+                des.update_search();
                 return Ok(CmdResult::Stay);
             }
         }
@@ -319,6 +439,13 @@ impl AppState {
             }
         }
 
+        // --- help
+
+        if key == F1 || key == QUESTION { // note that F1 is rarely available in terminals
+            self.help = Some(HelpState::default());
+            return Ok(CmdResult::Stay);
+        }
+
         // --- navigation among entries
 
         if let DrawerEdit(des) = &mut self.drawer_state {
@@ -344,11 +471,25 @@ impl AppState {
             if let DrawerEdit(des) = &mut self.drawer_state {
                 if let NameSelected { line } = &des.focus {
                     let line = *line;
-                    des.edit_entry_name_by_line(line);
+                    des.edit_entry_name_by_line(line, EditionPos::Start);
                 }
                 if let ValueSelected { line } = &des.focus {
                     let line = *line;
-                    des.edit_entry_value_by_line(line);
+                    des.edit_entry_value_by_line(line, EditionPos::Start);
+                }
+            }
+            return Ok(CmdResult::Stay);
+        }
+
+        if as_letter(key) == Some('a') {
+            if let DrawerEdit(des) = &mut self.drawer_state {
+                if let NameSelected { line } = &des.focus {
+                    let line = *line;
+                    des.edit_entry_name_by_line(line, EditionPos::End);
+                }
+                if let ValueSelected { line } = &des.focus {
+                    let line = *line;
+                    des.edit_entry_value_by_line(line, EditionPos::End);
                 }
             }
             return Ok(CmdResult::Stay);
@@ -359,6 +500,8 @@ impl AppState {
                 if let NameSelected { line } = &des.focus {
                     let line = *line;
                     des.focus = ValueSelected { line };
+                } else if matches!(des.focus, NoneSelected) {
+                    des.focus = NameSelected { line: 0 };
                 }
                 return Ok(CmdResult::Stay);
             }
@@ -366,83 +509,17 @@ impl AppState {
                 if let ValueSelected { line } = &des.focus {
                     let line = *line;
                     des.focus = NameSelected { line };
+                } else if matches!(des.focus, NoneSelected) {
+                    des.focus = NameSelected { line: 0 };
                 }
                 return Ok(CmdResult::Stay);
             }
             if key == UP {
-                if des.focus.is_search() {
-                    if let Some(line) = des.best_search_line() {
-                        let line = if line > 0 {
-                            line - 1
-                        } else {
-                            des.listed_entries_count() - 1
-                        };
-                        des.focus = NameSelected { line };
-                    } else {
-                        // there's no match, so there's no point to keep the search
-                        des.search.clear();
-                        des.search.update(&des.drawer);
-                        des.focus = NameSelected { line: 0 };
-                    }
-                    return Ok(CmdResult::Stay);
-                }
-                if let NameSelected { line } = &des.focus {
-                    let line = if *line > 0 {
-                        line - 1
-                    } else {
-                        des.listed_entries_count() - 1
-                    };
-                    des.focus = NameSelected { line };
-                }
-                if let ValueSelected { line } = &des.focus {
-                    let line = if *line > 0 {
-                        line - 1
-                    } else {
-                        des.listed_entries_count() - 1
-                    };
-                    des.focus = ValueSelected { line };
-                }
-                if matches!(des.focus, NoneSelected) {
-                    des.focus = NameSelected { line: 0 };
-                }
+                des.move_line(Direction::Up);
                 return Ok(CmdResult::Stay);
             }
             if key == DOWN {
-                if des.focus.is_search() {
-                    if let Some(line) = des.best_search_line() {
-                        let line = if line < des.listed_entries_count() {
-                            line + 1
-                        } else {
-                            0
-                        };
-                        des.focus = NameSelected { line };
-                    } else {
-                        // there's no match, so there's no point to keep the search
-                        des.search.clear();
-                        des.search.update(&des.drawer);
-                        des.focus = NameSelected { line: 0 };
-                    }
-                    return Ok(CmdResult::Stay);
-                }
-                if let NameSelected { line } = &des.focus {
-                    let line = if *line + 1 < des.listed_entries_count() {
-                        line + 1
-                    } else {
-                        0
-                    };
-                    des.focus = NameSelected { line };
-                }
-                if let ValueSelected { line } = &des.focus {
-                    let line = if *line + 1 < des.listed_entries_count() {
-                        line + 1
-                    } else {
-                        0
-                    };
-                    des.focus = ValueSelected { line };
-                }
-                if matches!(des.focus, NoneSelected) {
-                    des.focus = NameSelected { line: 0 };
-                }
+                des.move_line(Direction::Down);
                 return Ok(CmdResult::Stay);
             }
         }
@@ -458,7 +535,7 @@ impl AppState {
                         // new entry
                         des.search.clear();
                         let idx = des.drawer.content.empty_entry();
-                        des.edit_entry_name_by_line(idx);
+                        des.edit_entry_name_by_line(idx, EditionPos::Start);
                     }
                     ('d', Some(line)) => {
                         // delete entry (with confirmation)
