@@ -5,6 +5,7 @@ use {
 };
 
 /// State of the TUI application relative to drawers
+#[allow(clippy::large_enum_variant)]
 pub enum DrawerState {
     NoneOpen, // no drawer is open
     DrawerCreation(PasswordInputState),
@@ -12,7 +13,6 @@ pub enum DrawerState {
     DrawerEdit(DrawerEditState),
 }
 
-
 impl Default for DrawerState {
     fn default() -> Self {
         Self::NoneOpen
@@ -28,6 +28,12 @@ impl DrawerState {
     pub fn is_edit(&self) -> bool {
         matches!(self, DrawerState::DrawerEdit(_))
     }
+    pub fn is_pending_removal(&self) -> bool {
+        matches!(
+            self,
+            Self::DrawerEdit(DrawerEditState { focus: DrawerFocus::PendingRemoval{..}, .. }),
+        )
+    }
     pub fn input(&mut self) -> Option<&mut InputField> {
         match self {
             Self::DrawerCreation(PasswordInputState { input }) => Some(input),
@@ -42,6 +48,17 @@ impl DrawerState {
         }
     }
     #[allow(dead_code)]
+    pub fn is_on_entry_value(&mut self) -> bool {
+        match self {
+            Self::DrawerEdit(des) => match &mut des.focus {
+                DrawerFocus::NameEdit { .. } => true,
+                DrawerFocus::ValueEdit { .. } => true,
+                _ => false,
+            },
+            _ => false,
+        }
+    }
+    #[allow(dead_code)]
     pub fn has_input(&self) -> bool {
         match self {
             Self::DrawerCreation(PasswordInputState { .. }) => true,
